@@ -60,7 +60,7 @@ Suite *suite_create(const char *name)
 {
     Suite *s;
 
-    s = emalloc(sizeof(Suite)); /* freed in suite_free */
+    s = (Suite *)emalloc(sizeof(Suite)); /* freed in suite_free */
     if(name == NULL)
         s->name = "";
     else
@@ -80,7 +80,7 @@ int suite_tcase(Suite * s, const char *tcname)
     l = s->tclst;
     for(check_list_front(l); !check_list_at_end(l); check_list_advance(l))
     {
-        tc = check_list_val(l);
+        tc = (TCase *)check_list_val(l);
         if(strcmp(tcname, tc->name) == 0)
             return 1;
     }
@@ -97,7 +97,7 @@ static void suite_free(Suite * s)
     l = s->tclst;
     for(check_list_front(l); !check_list_at_end(l); check_list_advance(l))
     {
-        tcase_free(check_list_val(l));
+        tcase_free((TCase *)check_list_val(l));
     }
     check_list_free(s->tclst);
     free(s);
@@ -108,7 +108,7 @@ TCase *tcase_create(const char *name)
     char *env;
     double timeout_sec = DEFAULT_TIMEOUT;
 
-    TCase *tc = emalloc(sizeof(TCase)); /*freed in tcase_free */
+    TCase *tc = (TCase *)emalloc(sizeof(TCase)); /*freed in tcase_free */
 
     if(name == NULL)
         tc->name = "";
@@ -184,7 +184,7 @@ void _tcase_add_test(TCase * tc, TFun fn, const char *name, int _signal,
 
     if(tc == NULL || fn == NULL || name == NULL)
         return;
-    tf = emalloc(sizeof(TF));   /* freed in tcase_free */
+    tf = (TF *)emalloc(sizeof(TF));   /* freed in tcase_free */
     tf->fn = fn;
     tf->loop_start = start;
     tf->loop_end = end;
@@ -198,7 +198,7 @@ static Fixture *fixture_create(SFun fun, int ischecked)
 {
     Fixture *f;
 
-    f = emalloc(sizeof(Fixture));
+    f = (Fixture *)emalloc(sizeof(Fixture));
     f->fun = fun;
     f->ischecked = ischecked;
 
@@ -212,16 +212,7 @@ void tcase_add_unchecked_fixture(TCase * tc, SFun setup, SFun teardown)
 
 void tcase_add_checked_fixture(TCase * tc, SFun setup, SFun teardown)
 {
-#if defined(HAVE_FORK)
     tcase_add_fixture(tc, setup, teardown, 1);
-#else
-    (void)tc;
-    (void)setup;
-    (void)teardown;
-    eprintf
-        ("This version does not support checked fixtures, as fork is not supported",
-         __FILE__, __LINE__);
-#endif
 }
 
 static void tcase_add_fixture(TCase * tc, SFun setup, SFun teardown,
@@ -322,12 +313,12 @@ void _ck_assert_failed(const char *file, int line, const char *expr, ...)
 
 SRunner *srunner_create(Suite * s)
 {
-    SRunner *sr = emalloc(sizeof(SRunner));     /* freed in srunner_free */
+    SRunner *sr = (SRunner *)emalloc(sizeof(SRunner));     /* freed in srunner_free */
 
     sr->slst = check_list_create();
     if(s != NULL)
         check_list_add_end(sr->slst, s);
-    sr->stats = emalloc(sizeof(TestStats));     /* freed in srunner_free */
+    sr->stats = (TestStats *)emalloc(sizeof(TestStats));     /* freed in srunner_free */
     sr->stats->n_checked = sr->stats->n_failed = sr->stats->n_errors = 0;
     sr->resultlst = check_list_create();
     sr->log_fname = NULL;
@@ -368,14 +359,14 @@ void srunner_free(SRunner * sr)
     l = sr->slst;
     for(check_list_front(l); !check_list_at_end(l); check_list_advance(l))
     {
-        suite_free(check_list_val(l));
+        suite_free((Suite *)check_list_val(l));
     }
     check_list_free(sr->slst);
 
     l = sr->resultlst;
     for(check_list_front(l); !check_list_at_end(l); check_list_advance(l))
     {
-        tr = check_list_val(l);
+        tr = (TestResult *)check_list_val(l);
         tr_free(tr);
     }
     check_list_free(sr->resultlst);
@@ -399,13 +390,13 @@ TestResult **srunner_failures(SRunner * sr)
     TestResult **trarray;
     List *rlst;
 
-    trarray = malloc(sizeof(trarray[0]) * srunner_ntests_failed(sr));
+    trarray = (TestResult **)emalloc(sizeof(trarray[0]) * srunner_ntests_failed(sr));
 
     rlst = sr->resultlst;
     for(check_list_front(rlst); !check_list_at_end(rlst);
         check_list_advance(rlst))
     {
-        TestResult *tr = check_list_val(rlst);
+        TestResult *tr = (TestResult *)check_list_val(rlst);
 
         if(non_pass(tr->rtype))
             trarray[i++] = tr;
@@ -420,13 +411,13 @@ TestResult **srunner_results(SRunner * sr)
     TestResult **trarray;
     List *rlst;
 
-    trarray = malloc(sizeof(trarray[0]) * srunner_ntests_run(sr));
+    trarray =(TestResult **) emalloc(sizeof(trarray[0]) * srunner_ntests_run(sr));
 
     rlst = sr->resultlst;
     for(check_list_front(rlst); !check_list_at_end(rlst);
         check_list_advance(rlst))
     {
-        trarray[i++] = check_list_val(rlst);
+        trarray[i++] = (TestResult *)check_list_val(rlst);
     }
     return trarray;
 }
@@ -440,7 +431,7 @@ TestResult *tr_create(void)
 {
     TestResult *tr;
 
-    tr = emalloc(sizeof(TestResult));
+    tr = (TestResult *)emalloc(sizeof(TestResult));
     tr_init(tr);
     return tr;
 }
@@ -495,7 +486,7 @@ const char *tr_tcname(TestResult * tr)
     return tr->tcname;
 }
 
-static int _fstat = CK_FORK;
+static enum fork_status _fstat = CK_FORK;
 
 void set_fork_status(enum fork_status fstat)
 {
