@@ -63,10 +63,13 @@ static void srunner_iterate_suites(SRunner * sr,
                                    enum print_output print_mode);
 static void srunner_iterate_tcase_tfuns(SRunner * sr, TCase * tc);
 static void srunner_add_failure(SRunner * sr, TestResult * tf);
+static TestResult * srunner_run_setup(List * func_list,
+    enum fork_status fork_usage, const char * test_name,
+    const char * setup_name);
 static int srunner_run_unchecked_setup(SRunner * sr, TCase * tc);
 static TestResult *tcase_run_checked_setup(SRunner * sr, TCase * tc);
-static void srunner_run_teardown(List * l);
-static void srunner_run_unchecked_teardown(TCase * tc);
+static void srunner_run_teardown(List * fixture_list, enum fork_status fork_usage);
+static void srunner_run_unchecked_teardown(SRunner * sr, TCase * tc);
 static void tcase_run_checked_teardown(TCase * tc);
 static void srunner_run_tcase(SRunner * sr, TCase * tc);
 static TestResult *tcase_run_tfun_nofork(SRunner * sr, TCase * tc, TF * tf,
@@ -142,7 +145,7 @@ static void srunner_iterate_suites(SRunner * sr,
     for(check_list_front(slst); !check_list_at_end(slst);
         check_list_advance(slst))
     {
-        Suite *s = check_list_val(slst);
+        Suite *s = (Suite *)check_list_val(slst);
 
         if(((sname != NULL) && (strcmp(sname, s->name) != 0))
            || ((tcname != NULL) && (!suite_tcase(s, tcname))))
@@ -155,7 +158,7 @@ static void srunner_iterate_suites(SRunner * sr,
         for(check_list_front(tcl); !check_list_at_end(tcl);
             check_list_advance(tcl))
         {
-            tc = check_list_val(tcl);
+            tc = (TCase *)check_list_val(tcl);
 
             if((tcname != NULL) && (strcmp(tcname, tc->name) != 0))
             {
@@ -182,7 +185,7 @@ static void srunner_iterate_tcase_tfuns(SRunner * sr, TCase * tc)
     {
         int i;
 
-        tfun = check_list_val(tfl);
+        tfun = (TF *)check_list_val(tfl);
 
         for(i = tfun->loop_start; i < tfun->loop_end; i++)
         {
@@ -225,79 +228,33 @@ static void srunner_add_failure(SRunner * sr, TestResult * tr)
 
 }
 
-static int srunner_run_unchecked_setup(SRunner * sr, TCase * tc)
-{
-    TestResult *tr;
-    List *l;
-    Fixture *f;
-    int rval = 1;
-
-    set_fork_status(CK_NOFORK);
-
-    l = tc->unch_sflst;
-
-    for(check_list_front(l); !check_list_at_end(l); check_list_advance(l))
-    {
-        send_ctx_info(CK_CTX_SETUP);
-        f = check_list_val(l);
-
-        if(0 == setjmp(error_jmp_buffer))
-        {
-            f->fun();
-        }
-
-        tr = receive_result_info_nofork(tc->name, "unchecked_setup", 0, -1);
-
-        if(tr->rtype != CK_PASS)
-        {
-            srunner_add_failure(sr, tr);
-            rval = 0;
-            break;
-        }
-        free(tr->file);
-        free(tr->msg);
-        free(tr);
-    }
-
-    set_fork_status(srunner_fork_status(sr));
-    return rval;
-}
-
-static TestResult *tcase_run_checked_setup(SRunner * sr, TCase * tc)
+static TestResult * srunner_run_setup(List * fixture_list, enum fork_status fork_usage,
+    const char * test_name, const char * setup_name)
 {
     TestResult *tr = NULL;
-    List *l;
-    Fixture *f;
-    enum fork_status fstat = srunner_fork_status(sr);
+    Fixture *setup_fixture;
 
-    l = tc->ch_sflst;
-    if(fstat == CK_FORK)
+    if(fork_usage == CK_FORK)
     {
         send_ctx_info(CK_CTX_SETUP);
     }
 
-    for(check_list_front(l); !check_list_at_end(l); check_list_advance(l))
+    for(check_list_front(fixture_list); !check_list_at_end(fixture_list);
+        check_list_advance(fixture_list))
     {
-        f = check_list_val(l);
+        setup_fixture = (Fixture *)check_list_val(fixture_list);
 
-        if(fstat == CK_NOFORK)
+        if(fork_usage == CK_NOFORK)
         {
             send_ctx_info(CK_CTX_SETUP);
 
             if(0 == setjmp(error_jmp_buffer))
             {
-                f->fun();
+                setup_fixture->fun();
             }
-        }
-        else
-        {
-            f->fun();
-        }
 
-        /* Stop the setup and return the failure if nofork mode. */
-        if(fstat == CK_NOFORK)
-        {
-            tr = receive_result_info_nofork(tc->name, "checked_setup", 0, -1);
+            /* Stop the setup and return the failure in nofork mode. */
+            tr = receive_result_info_nofork(test_name, setup_name, 0, -1);
             if(tr->rtype != CK_PASS)
             {
                 break;
@@ -308,31 +265,78 @@ static TestResult *tcase_run_checked_setup(SRunner * sr, TCase * tc)
             free(tr);
             tr = NULL;
         }
+        else
+        {
+            setup_fixture->fun();
+        }
     }
 
     return tr;
 }
 
-static void srunner_run_teardown(List * l)
+static int srunner_run_unchecked_setup(SRunner * sr, TCase * tc)
 {
-    Fixture *f;
+    TestResult *tr = NULL;
+    int rval = 1;
 
-    for(check_list_front(l); !check_list_at_end(l); check_list_advance(l))
+    set_fork_status(CK_NOFORK);
+    tr = srunner_run_setup(tc->unch_sflst, CK_NOFORK, tc->name, "unchecked_setup");
+    set_fork_status(srunner_fork_status(sr));
+
+    if(tr != NULL && tr->rtype != CK_PASS)
     {
-        f = check_list_val(l);
+        srunner_add_failure(sr, tr);
+        rval = 0;
+    }
+
+    return rval;
+}
+
+static TestResult *tcase_run_checked_setup(SRunner * sr, TCase * tc)
+{
+    TestResult *tr = srunner_run_setup(tc->ch_sflst, srunner_fork_status(sr),
+        tc->name, "checked_setup");
+
+    return tr;
+}
+
+static void srunner_run_teardown(List * fixture_list, enum fork_status fork_usage)
+{
+    Fixture * fixture;
+
+    for(check_list_front(fixture_list); !check_list_at_end(fixture_list);
+        check_list_advance(fixture_list))
+    {
+        fixture = (Fixture *)check_list_val(fixture_list);
         send_ctx_info(CK_CTX_TEARDOWN);
-        f->fun();
+
+        if(fork_usage == CK_NOFORK)
+        {
+            if(0 == setjmp(error_jmp_buffer))
+            {
+                fixture->fun();
+            }
+            else
+            {
+                /* Abort the remaining teardowns */
+                break;
+            }
+        }
+        else
+        {
+            fixture->fun();
+        }
     }
 }
 
-static void srunner_run_unchecked_teardown(TCase * tc)
+static void srunner_run_unchecked_teardown(SRunner * sr, TCase * tc)
 {
-    srunner_run_teardown(tc->unch_tflst);
+    srunner_run_teardown(tc->unch_tflst, srunner_fork_status(sr));
 }
 
 static void tcase_run_checked_teardown(TCase * tc)
 {
-    srunner_run_teardown(tc->ch_tflst);
+    srunner_run_teardown(tc->ch_tflst, CK_NOFORK);
 }
 
 static void srunner_run_tcase(SRunner * sr, TCase * tc)
@@ -340,7 +344,7 @@ static void srunner_run_tcase(SRunner * sr, TCase * tc)
     if(srunner_run_unchecked_setup(sr, tc))
     {
         srunner_iterate_tcase_tfuns(sr, tc);
-        srunner_run_unchecked_teardown(tc);
+        srunner_run_unchecked_teardown(sr, tc);
     }
 }
 
@@ -348,9 +352,7 @@ static TestResult *tcase_run_tfun_nofork(SRunner * sr, TCase * tc, TF * tfun,
                                          int i)
 {
     TestResult *tr;
-    struct timespec ts_start = { 0, 0 }, ts_end =
-    {
-    0, 0};
+    struct timespec ts_start = {0, 0}, ts_end = {0, 0};
 
     tr = tcase_run_checked_setup(sr, tc);
     if(tr == NULL)
@@ -417,9 +419,7 @@ static TestResult *tcase_run_tfun_fork(SRunner * sr, TCase * tc, TF * tfun,
     pid_t pid_w;
     pid_t pid;
     int status = 0;
-    struct timespec ts_start = { 0, 0 }, ts_end =
-    {
-    0, 0};
+    struct timespec ts_start = { 0, 0 }, ts_end ={ 0, 0 };
 
     timer_t timerid;
     struct itimerspec timer_spec;
@@ -608,7 +608,7 @@ static void set_fork_info(TestResult * tr, int status, int signal_expected,
 
 static char *signal_msg(int signal)
 {
-    char *msg = emalloc(MSG_LEN);       /* free'd by caller */
+    char *msg = (char *)emalloc(MSG_LEN);       /* free'd by caller */
 
     if(alarm_received)
     {
@@ -626,7 +626,7 @@ static char *signal_error_msg(int signal_received, int signal_expected)
 {
     char *sig_r_str;
     char *sig_e_str;
-    char *msg = emalloc(MSG_LEN);       /* free'd by caller */
+    char *msg = (char *)emalloc(MSG_LEN);       /* free'd by caller */
 
     sig_r_str = strdup(strsignal(signal_received));
     sig_e_str = strdup(strsignal(signal_expected));
@@ -648,7 +648,7 @@ static char *signal_error_msg(int signal_received, int signal_expected)
 
 static char *exit_msg(int exitval)
 {
-    char *msg = emalloc(MSG_LEN);       /* free'd by caller */
+    char *msg = (char *)emalloc(MSG_LEN);       /* free'd by caller */
 
     snprintf(msg, MSG_LEN, "Early exit with return value %d", exitval);
     return msg;
